<?php
/**
 * Gestions des plugins
 * @author 		neolao <neo@neolao.com>
 */
class PluginManager {
	/**
	 * La liste des plugins actifs
	 */
	public $list = false;
	/**
	 * la liste des plugins désactivés
	 */
	public $disabled;
	/**
	 * Le chemin du répertoire plugins
	 */
	public $path;
	
	// ============================== CONSTRUCTOR ==============================
	/**
	 * Constructeur
	 * @param $path Le chemin du répertoire plugins
	 */
	function PluginManager($path){		
		$this->path = $path;
		$this->_init();
		$this->_check();
	}
	
	// ================================ PRIVATE ================================
	/**
	 * Initialisation
	 */
	private function _init(){
		// Liste des plugins actifs
		$this->list = array();
		try {
		    $dir = new directoryIterator($this->path);
		} catch (Exception $e) {
			$error->add($e);
		}
		foreach($dir as $value) {
	    	$dirName = (string) $value;
	    	
	    	// Pour que ce soit un vrai plugin, il faut un fichier config.ini à la racine
	    	if(!$value->isDot() && is_file($this->path.$dirName."/config.ini")){
				$this->list[$dirName] = array();
	    	}
		}
		
		// Liste des plugins désactivés
		$data = new DataText(WM_DATAS_PATH."PluginManager.txt");
		$dataContent = $data->get();
		if(empty($dataContent)){
			$data->set("disabled = \n");
			$this->disabled = array();
		}else{
			$dataIni = @parse_ini_file(WM_DATAS_PATH."PluginManager.txt");
			$this->disabled = split(",", $dataIni["disabled"]);			
		}
	}
	/**
	 * Vérification de la liste des plugins
	 */
	private function _check(){
		global $error, $lang, $CONFIG, $updates;
		
		reset($this->list);
		foreach($this->list as $dirName=>$plugin){
    		// On vérifie que l'extension n'est pas désactivées
    		$this->list[$dirName]["disabled"] = in_array($dirName, $this->disabled);
			
			// Chargement du fichier de langue
			$plugin_lang = new Language($this->path.$dirName."/lang/", $CONFIG["general"]["lang_default"], $CONFIG["general"]["encoding"]);
    		$this->list[$dirName]["lang"] = $plugin_lang;
			
			if(!$this->list[$dirName]["disabled"]){
				// Chargement du fichier config.ini
				$config = @parse_ini_file($this->path.$dirName."/config.ini");
				if(!$config){
					$error->add($lang->get("warning", "read")." : ".$this->path.$dirName."/config.ini");
				}else{
					foreach($config as $key=>$value){
						$this->list[$dirName][$key] = $value;
					}
				}
				$config["dirName"] = $dirName;
				
				// Mise à jour
				if(!empty($config["updateType"]) && !empty($config["updateUrl"])){
					switch($config["updateType"]){
						case "archive":
							$update = new UpdateFromArchive($plugin_lang->get("menu", "title"), $this->path.$dirName, $config["updateUrl"]);
							break;
						case "xml":
							$update = new UpdateFromXML($plugin_lang->get("menu", "title"), $this->path.$dirName, $config["updateUrl"]);
							break;
					}
					array_push($updates, $update);
				}
	    		
	    		// Instanciation de la class
	    		if(!empty($config["class"]) && file_exists($this->path.$dirName."/class.php")){
	    			require_once $this->path.$dirName."/class.php";
	    			if(class_exists($config["class"])){
		    			$this->list[$dirName]["class"] = new $config["class"];
		    			$this->list[$dirName]["class"]->addConfig($config);
		    			$this->list[$dirName]["class"]->addLang($plugin_lang);
		    			$this->list[$dirName]["class"]->init();
	    			}else{
	    				array_pop($this->list);
	    				$error->add($lang->get("warning", "class_miss")." : ".$this->path.$dirName);
	    			}
	    		}else{
	    			$error->add($lang->get("warning", "class_miss")." : ".$this->path.$dirName);
	    		}
			}
		}
		ksort($this->list);
	}
	/**
	 * Ajouter les plugins au menu
	 */
	private function _addToMenu(){
		global $menu;
		global $lang;
		$category = $lang->get("menu", "plugins");
		
		if(count($this->list) > 0){
			$menu->addCategory($category);
			reset($this->list);
			foreach($this->list as $plugin=>$value){
				if(!$value["disabled"]){
					$menu->addLink($category, $value["lang"]->get("menu", "title"), WM_PAGE."?id=plugin:".$plugin);
				}
			}
		}
	}
	/**
	 * Enregistrement des données de l'administration
	 */
	private function _saveAdmin(){
		$this->disabled = array();
		
		reset($this->list);
		foreach($this->list as $plugin=>$value){
			if(!empty($_POST["disable_".$plugin])){
				array_push($this->disabled, $plugin);
			}
		}
		
		$data = new DataText(WM_DATAS_PATH."PluginManager.txt");
		$data->set("disabled = ".implode(",", $this->disabled)."\n");
	}
	/**
	 * Affichage de l'administration
	 */
	private function _showAdmin(){
		global $content;
		global $lang;
		
		$content->output .= "<h3>".$lang->get("admin", "pluginmanager_disabled")."</h3>";
		
		if(count($this->list) > 0){
			$form = new Form();
			$form->addHidden("id", "admin");
			$form->addHidden("category", "pluginmanager");

			reset($this->list);
			foreach($this->list as $plugin=>$value){
				$form->addCheckbox("disable_".$plugin, $value["lang"]->get("menu", "title"), $value["disabled"]);
			}
			
			$form->setSubmit($lang->get("general", "save"));
			$content->output .= $form->html();
		}
	}
	// ================================ PUBLIC =================================
	/**
	 * Exécution des plugins
	 */
	public function main(){
		global $content;
		
		// Enregistrement de l'administration
		if($content->id == "admin" && $_POST["category"] == "pluginmanager"){
			$this->_saveAdmin();
		}
		$this->_check();
		$this->_addToMenu();
		if($content->id == "admin"){
			$this->_showAdmin();
		}
		
		foreach($this->list as $plugin){
			if(!$plugin["disabled"]){
				$plugin["class"]->main();
			}
		}
	}
}
?>