<?php
/**
 * Mise à jour à partir d'une archive
 * 
 * L'archive est téléchargée puis décompressée.
 * Il y a ensuite une vérification des versions.
 * Si la version téléchargée est plus récente, elle écrase l'ancienne.
 * @author 		neolao <neo@neolao.com>
 */
class UpdateFromArchive extends AUpdate{
	/**
	 * L'url de l'archive
	 * @var string
	 */
	protected $_archive;
	/**
	 * Le chemin de l'archive téléchargée
	 * @var string
	 */
	protected $_archiveFile;
	/**
	 * Le chemin des fichiers temporaires de l'archive
	 * @var string
	 */
	protected $_temporyFiles;
	
	// ============================== CONSTRUCTOR ==============================
	/**
	 * Constructeur
	 * @param string $title Le titre de la mise à jour
	 * @param string $directory Le chemin du répertoire à mettre à jour
	 * @param string $archive L'url de l'archive contenant la dernière version
	 */
	function __construct($title, $directory, $archive){
		parent::__construct($title, $directory);
		$this->_archive = $archive;
	}
	// ================================ PRIVATE ================================
	/**
	 * Vide un répertoire
	 * @param string $folder Le répertoire à vider
	 */
	private function _cleanFolder($folder){
		if(is_dir($folder)){
			$list = $this->_listDir($folder);
			for($i=0; $i<count($list); $i++){
				if(!unlink($list[$i])){
					return false;
				}
			}
			return true;
		}else{
			return false;
		}
	}
	/**
	 * Téléchargement de l'archive
	 */
	protected function _getArchive(){
		global $error, $lang;
		
		$archiveName = basename($this->_archive);
		if($archive = @fopen($this->_archive, "r")){
			$archiveContent = "";
			while(!feof($archive)){
				$archiveContent .= fgets($archive, 4096);
			}
			fclose($archive);
			
			$this->_archiveFile = WM_TMP_PATH.$archiveName;
			if($file = @fopen($this->_archiveFile, "w")){
				fwrite($file, $archiveContent);
				fclose($file);
			}else{
				$error->add($lang->get("warning", "write")." : ".WM_TMP_PATH.$archiveName);
			}
		}else{
			$error->add($lang->get("warning", "read")." : ".$this->_archive);
		}
	}
	/**
	 * Décompresser l'archive
	 * @return true si l'opération s'est bien passée, sinon false
	 */
	protected function _uncompress(){
		global $error, $lang;
		
		if(file_exists($this->_archiveFile)){
			// créer le répertoire temporaire
			$this->_temporyFiles = $this->_archiveFile."_FILES";
			if(!is_dir($this->_temporyFiles) && !@mkdir($this->_temporyFiles, 0777)){
				$error->add($lang->get("warning", "write")." : ".$this->_temporyFiles);
				return false;
			}
			
			// Vide le répertoire temporaire
			if(!$this->_cleanFolder($this->_temporyFiles)){
				$error->add($lang->get("warning", "write")." : ".$this->_temporyFiles);
				return false;
			}
			
			// ----------------------------------------------------- méthode zip
			if(function_exists("zip_open")){
				$zip = @zip_open($this->_archiveFile);
				if($zip){
					while($zip_entry = zip_read($zip)){
						if(zip_entry_open($zip, $zip_entry, "r")){
							$fileName = zip_entry_name($zip_entry);
							
							// On vérifie si on traite un répertoire ou un fichier
							if(substr($fileName, -1) != "/"){
								// On traite un fichier
								
								// répertoire du fichier						
								$dir = dirname($this->_temporyFiles."/".$fileName);
								if(!is_dir($dir) && !@mkdir($dir, 0777, true)){
									$error->add($lang->get("warning", "write")." : ".$dir);
									return false;
								}
								
								// Contenu du fichier
								$content = zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));
								zip_entry_close($zip_entry);
								
								// Création du fichier
								if($file = @fopen($this->_temporyFiles."/".$fileName, "w")){
									fwrite($file, $content);
									fclose($file);
								}else{
									$error->add($lang->get("warning", "write")." : ".$this->_temporyFiles."/".$fileName);
									return false;
								}
							}else{
								// On traite un répertoire
								$dir = $this->_temporyFiles."/".$fileName;
								if(!is_dir($dir) && !@mkdir($dir, 0777, true)){
									$error->add($lang->get("warning", "write")." : ".$dir);
									return false;
								}
							}
						}
					}
					zip_close($zip);
					return true;
				}
			}else{
				$error->add($lang->get("warning", "function_zip")." : ".$this->_archiveFile);
				return false;
			}
		}
		return false;
	}
	/**
	 * Récupération de la dernière version
	 */
	protected function _getLatestVersion(){
		global $error, $lang;
		$configFile = $this->_temporyFiles."/config.ini";
		
		if(file_exists($configFile)){
			// Le fichier config.ini existe
			// On le parse
			$configContent = parse_ini_file($configFile);
			if(!empty($configContent["version"])){
				// Récupération de la dernière version
				$this->_latestVersion = $configContent["version"];
			}else{
				// Récupération de la dernière version impossible
				$error->add($lang->get("warning", "version_miss")." : ".$configFile);
			}
		}else{
			// Le fichier config.ini n'existe pas
			$error->add($lang->get("warning", "file_miss")." : ".$configFile);
		}
	}
	/**
	 * Récupération de la liste des fichiers à mettre à jour
	 */
	protected function _getNewFiles(){
		if(is_dir($this->_temporyFiles)){
			$list = $this->_listDir($this->_temporyFiles);
			
			// Remplissage de la liste
			$this->_newFiles = array();
			for($i=0; $i<count($list); $i++){
				// On transforme le chemin absolu en chemin relatif au dossier temporaire
				$relativePath = substr($list[$i], strlen($this->_temporyFiles));
				
				$file = array();
				$file["src"] = $list[$i];
				$file["target"] = $this->_dir.$relativePath;
				
				array_push($this->_newFiles, $file);
			}
		}
	}
	/**
	 * Lister les fichiers d'un dossier
	 * @param string $dir Le répertoire à lister
	 * @return array La liste des fichiers
	 */
	private function _listDir($dir, $list=array()){
		$l = array(".", "..");
		
		if($d = @opendir($dir)){
			while($f = readdir($d)){
				if(!in_array($f, $l)){
					if(is_dir($dir."/".$f)){
						$list = $this->_listDir($dir."/".$f, $list);
					}else{
						array_push($list, $dir."/".$f);
					}
				}
			}
			return $list;
		}else{
			return false;
		}
	}
	// ================================ PUBLIC =================================
	/**
	 * Le titre de la mettre à jour
	 * @return Le titre
	 */
	public function getTitle(){
		return parent::getTitle();
	}
	/**
	 * Le titre de la mettre à jour
	 * @param string $title Le titre
	 */
	public function setTitle($title){
		parent::setTitle($title);
	}
	/**
	 * Le répertoire à mettre à jour
	 * @return Le répêrtoire
	 */
	public function getDirectory(){
		return parent::getDirectory();
	}
	/**
	 * Le répertoire à mettre à jour
	 * @param string $directory Le répertoire
	 */
	public function setDirectory($directory){
		parent::setDirectory($directory);
	}
	/**
	 * Récupération des informations utiles.
	 * C'est ici qu'on prépare la mise à jour
	 */
	public function check(){
		// Téléchargement de l'archive
		$this->_getArchive();
		
		// Décompression
		$this->_uncompress();
		
		// Récupération de la version installée
		parent::check();
		
		// Récupération de la version téléchargée
		$this->_getLatestVersion();
		
		// La liste des fichiers à mettre à jour
		$this->_getNewFiles();
	}
	/**
	 * Récupèrè la version installée
	 * @return string La version installée
	 */
	public function getCurrentVersion(){
		return parent::getCurrentVersion();
	}
	/**
	 * Récupère la dernière version officielle
	 * @return string La dernière version officielle
	 */
	public function getLatestVersion(){
		return parent::getLatestVersion();
	}
	/**
	 * Récupère la liste des fichiers à mettre à jour
	 * @return array La liste des fichiers à mettre à jour
	 */
	public function getNewFiles(){
		return parent::getNewFiles();
	}
	/**
	 * Met à jour
	 */
	public function update(){
		parent::update();
	}
}
?>